/*********************************************************************/
/* Copyright (c) 2013, EPFL/Blue Brain Project                       */
/*                     Raphael Dumusc <raphael.dumusc@epfl.ch>       */
/* All rights reserved.                                              */
/*                                                                   */
/* Redistribution and use in source and binary forms, with or        */
/* without modification, are permitted provided that the following   */
/* conditions are met:                                               */
/*                                                                   */
/*   1. Redistributions of source code must retain the above         */
/*      copyright notice, this list of conditions and the following  */
/*      disclaimer.                                                  */
/*                                                                   */
/*   2. Redistributions in binary form must reproduce the above      */
/*      copyright notice, this list of conditions and the following  */
/*      disclaimer in the documentation and/or other materials       */
/*      provided with the distribution.                              */
/*                                                                   */
/*    THIS  SOFTWARE IS PROVIDED  BY THE  UNIVERSITY OF  TEXAS AT    */
/*    AUSTIN  ``AS IS''  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,    */
/*    INCLUDING, BUT  NOT LIMITED  TO, THE IMPLIED  WARRANTIES OF    */
/*    MERCHANTABILITY  AND FITNESS FOR  A PARTICULAR  PURPOSE ARE    */
/*    DISCLAIMED.  IN  NO EVENT SHALL THE UNIVERSITY  OF TEXAS AT    */
/*    AUSTIN OR CONTRIBUTORS BE  LIABLE FOR ANY DIRECT, INDIRECT,    */
/*    INCIDENTAL,  SPECIAL, EXEMPLARY,  OR  CONSEQUENTIAL DAMAGES    */
/*    (INCLUDING, BUT  NOT LIMITED TO,  PROCUREMENT OF SUBSTITUTE    */
/*    GOODS  OR  SERVICES; LOSS  OF  USE,  DATA,  OR PROFITS;  OR    */
/*    BUSINESS INTERRUPTION) HOWEVER CAUSED  AND ON ANY THEORY OF    */
/*    LIABILITY, WHETHER  IN CONTRACT, STRICT  LIABILITY, OR TORT    */
/*    (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT    */
/*    OF  THE  USE OF  THIS  SOFTWARE,  EVEN  IF ADVISED  OF  THE    */
/*    POSSIBILITY OF SUCH DAMAGE.                                    */
/*                                                                   */
/* The views and conclusions contained in the software and           */
/* documentation are those of the authors and should not be          */
/* interpreted as representing official policies, either expressed   */
/* or implied, of The University of Texas at Austin.                 */
/*********************************************************************/

#ifndef MASTERCONFIGURATION_H
#define MASTERCONFIGURATION_H

#include "Configuration.h"

class QXmlQuery;

/**
 * @brief The MasterConfiguration class manages all the parameters needed
 * to setup the Master process.
 */
class MasterConfiguration : public Configuration
{
public:
    /**
     * @brief MasterConfiguration constructor
     * @param filename \see Configuration
     * @throw std::runtime_error if the file could not be read
     */
    MasterConfiguration(const QString& filename);

    /**
     * @brief getDockStartDir Get the Dock startup directory
     * @return directory path
     */
    const QString& getDockStartDir() const;

    /**
     * @brief getWebServicePort Get the port where the WebService server
     * will be listening for incoming requests.
     * @return port for WebService server
     */
    int getWebServicePort() const;

    /**
     * @brief Get the URL used as start page when opening a Web Browser.
     * @return The URL defined in the configuration file, or a default value if
     * none is found.
     */
    const QString& getWebBrowserDefaultURL() const;

private:
    void loadMasterSettings();
    void loadDockStartDirectory(QXmlQuery& query);
    void loadWebBrowserStartURL(QXmlQuery& query);

    QString dockStartDir_;
    int dcWebServicePort_;
    QString webBrowserDefaultURL_;
};

#endif // MASTERCONFIGURATION_H
